@@ -1,13 +1,12 @@
 //! Provides a utility for generating `enum_dispatch` impl blocks given `EnumDispatchItem` and
 //! `syn::ItemTrait` definitions.
 use crate::{cache, proc_macro};
-use proc_macro2;
 use quote::{quote, ToTokens};
-use syn;
 use syn::spanned::Spanned;
 
 use crate::enum_dispatch_item::EnumDispatchItem;
 use crate::enum_dispatch_variant::EnumDispatchVariant;
+use crate::syn_utils::plain_identifier_expr;
 
 /// Name bound to the single enum field in generated match statements. It doesn't really matter
 /// what this is, as long as it's consistent across the left and right sides of generated match
@@ -23,7 +22,11 @@ pub fn add_enum_impls(
     let traitname = traitdef.ident;
     let traitfns = traitdef.items;
 
-    let generics = if traitdef.generics.lt_token.is_some() { &traitdef.generics } else { &enum_def.generics };
+    let generics = if traitdef.generics.lt_token.is_some() {
+        &traitdef.generics
+    } else {
+        &enum_def.generics
+    };
     let (_, enum_ty, _) = &traitdef.generics.split_for_impl();
     let (impl_generics, ty_generics, where_clause) = generics.split_for_impl();
     let enumname = &enum_def.ident.to_owned();
@@ -39,9 +42,12 @@ pub fn add_enum_impls(
     let variants: Vec<&EnumDispatchVariant> = enum_def.variants.iter().collect();
 
     for trait_fn in traitfns {
-        trait_impl
-            .items
-            .push(create_trait_match(trait_fn, &enum_def.ident, &variants));
+        trait_impl.items.push(create_trait_match(
+            trait_fn,
+            &traitname,
+            &enum_def.ident,
+            &variants,
+        ));
     }
 
     let mut impls = proc_macro2::TokenStream::new();
@@ -53,7 +59,8 @@ pub fn add_enum_impls(
             from_impl.to_tokens(&mut impls);
         }
 
-        let try_into_impls = generate_try_into_impls(&enum_def.ident, &variants, &trait_impl.generics);
+        let try_into_impls =
+            generate_try_into_impls(&enum_def.ident, &variants, &trait_impl.generics);
         for try_into_impl in try_into_impls.iter() {
             try_into_impl.to_tokens(&mut impls);
         }
@@ -65,7 +72,11 @@ pub fn add_enum_impls(
 }
 
 /// Generates impls of core::convert::From for each enum variant.
-fn generate_from_impls(enumname: &syn::Ident, enumvariants: &[&EnumDispatchVariant], generics: &syn::Generics) -> Vec<syn::ItemImpl> {
+fn generate_from_impls(
+    enumname: &syn::Ident,
+    enumvariants: &[&EnumDispatchVariant],
+    generics: &syn::Generics,
+) -> Vec<syn::ItemImpl> {
     let (impl_generics, ty_generics, where_clause) = generics.split_for_impl();
     enumvariants
         .iter()
@@ -84,7 +95,11 @@ fn generate_from_impls(enumname: &syn::Ident, enumvariants: &[&EnumDispatchVaria
 }
 
 /// Generates impls of core::convert::TryInto for each enum variant.
-fn generate_try_into_impls(enumname: &syn::Ident, enumvariants: &[&EnumDispatchVariant], generics: &syn::Generics) -> Vec<syn::ItemImpl> {
+fn generate_try_into_impls(
+    enumname: &syn::Ident,
+    enumvariants: &[&EnumDispatchVariant],
+    generics: &syn::Generics,
+) -> Vec<syn::ItemImpl> {
     let (impl_generics, ty_generics, where_clause) = generics.split_for_impl();
     enumvariants
         .iter()
@@ -145,11 +160,15 @@ fn extract_fn_args(
     let new_args: Vec<syn::Ident> = trait_args
         .iter()
         .filter_map(|arg| match arg {
-            syn::FnArg::Receiver(syn::Receiver { reference: Some(_), .. }) => {
+            syn::FnArg::Receiver(syn::Receiver {
+                reference: Some(_), ..
+            }) => {
                 method_type = MethodType::ByReference;
                 None
             }
-            syn::FnArg::Receiver(syn::Receiver { reference: None, .. }) => {
+            syn::FnArg::Receiver(syn::Receiver {
+                reference: None, ..
+            }) => {
                 method_type = MethodType::ByValue;
                 None
             }
@@ -160,7 +179,8 @@ fn extract_fn_args(
                     panic!("Unsupported argument type")
                 }
             }
-        }).collect();
+        })
+        .collect();
     let args = {
         let mut args = syn::punctuated::Punctuated::new();
         new_args.iter().for_each(|arg| {
@@ -173,9 +193,16 @@ fn extract_fn_args(
 
 /// Creates a method call that can be used in the match arms of all non-static method
 /// implementations.
-fn create_trait_fn_call(trait_method: &syn::TraitItemMethod) -> syn::ExprCall {
+fn create_trait_fn_call(
+    trait_method: &syn::TraitItemMethod,
+    trait_name: &syn::Ident,
+) -> syn::ExprCall {
     let trait_args = trait_method.to_owned().sig.inputs;
-    let (method_type, args) = extract_fn_args(trait_args);
+    let (method_type, mut args) = extract_fn_args(trait_args);
+
+    // Insert FIELDNAME at the beginning of the argument list for UCFS-style method calling
+    let explicit_self_arg = syn::Ident::new(FIELDNAME, trait_method.span());
+    args.insert(0, plain_identifier_expr(explicit_self_arg));
 
     syn::ExprCall {
         attrs: vec![],
@@ -191,9 +218,8 @@ fn create_trait_fn_call(trait_method: &syn::TraitItemMethod) -> syn::ExprCall {
                     "Static methods cannot be enum_dispatched (no self argument to match on)"
                 );
             } else {
-                let fieldname = syn::Ident::new(FIELDNAME, trait_method.span());
                 let trait_method_name = &trait_method.sig.ident;
-                Box::new(syn::parse_quote! { #fieldname.#trait_method_name })
+                Box::new(syn::parse_quote! { #trait_name::#trait_method_name })
             }
         },
         paren_token: Default::default(),
@@ -205,10 +231,11 @@ fn create_trait_fn_call(trait_method: &syn::TraitItemMethod) -> syn::ExprCall {
 /// binding to their single field and calling the provided trait method on each.
 fn create_match_expr(
     trait_method: &syn::TraitItemMethod,
+    trait_name: &syn::Ident,
     enum_name: &syn::Ident,
     enumvariants: &[&EnumDispatchVariant],
 ) -> syn::Expr {
-    let trait_fn_call = create_trait_fn_call(trait_method);
+    let trait_fn_call = create_trait_fn_call(trait_method, trait_name);
 
     // Creates a Vec containing a match arm for every enum variant
     let match_arms = enumvariants
@@ -226,27 +253,17 @@ fn create_match_expr(
                 body: Box::new(syn::Expr::from(trait_fn_call.to_owned())),
                 comma: Some(Default::default()),
             }
-        }).collect();
+        })
+        .collect();
 
     // Creates the match expression
     syn::Expr::from(syn::ExprMatch {
         attrs: vec![],
         match_token: Default::default(),
-        expr: Box::new(syn::Expr::from(syn::ExprPath {
-            attrs: vec![],
-            qself: None,
-            path: syn::Path {
-                leading_colon: None,
-                segments: {
-                    let mut segments = syn::punctuated::Punctuated::new();
-                    segments.push(syn::PathSegment {
-                        ident: syn::Ident::new("self", syn::export::Span::call_site()),
-                        arguments: syn::PathArguments::None,
-                    });
-                    segments
-                },
-            },
-        })),
+        expr: Box::new(plain_identifier_expr(syn::Ident::new(
+            "self",
+            syn::export::Span::call_site(),
+        ))),
         brace_token: Default::default(),
         arms: match_arms,
     })
@@ -255,12 +272,13 @@ fn create_match_expr(
 /// Builds an implementation of the given trait function for the given enum type.
 fn create_trait_match(
     trait_item: syn::TraitItem,
+    trait_name: &syn::Ident,
     enum_name: &syn::Ident,
     enumvariants: &[&EnumDispatchVariant],
 ) -> syn::ImplItem {
     match trait_item {
         syn::TraitItem::Method(trait_method) => {
-            let match_expr = create_match_expr(&trait_method, enum_name, enumvariants);
+            let match_expr = create_match_expr(&trait_method, trait_name, enum_name, enumvariants);
 
             syn::ImplItem::Method(syn::ImplItemMethod {
                 attrs: vec![syn::Attribute {
